#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import argparse
import numpy as np

from model.sjyhwrd import Sjyhwrd
from model.init import scenario_init, age_groups_init, parameters_init, conditions_category_init, lengths_init
from analysis.utils_plot import plot_

MIN_0 = 0.0
MAX_1 = 1.0


def float_range_0_1_type(arg):
    try:
        f = float(arg)
    except ValueError:
        raise argparse.ArgumentTypeError("Must be a floating point number")
    if f < MIN_0 or f > MAX_1:
        raise argparse.ArgumentTypeError("Argument must be <= " + str(MAX_1) + "and >= " + str(MIN_0))
    return f


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Run Alizon and Al. Model')

    # Model parameters
    parser.add_argument("zeta_weibull_k", help="Generation time distribution - k", type=float, default=2.24)
    parser.add_argument("zeta_weibull_lbd", help="Generation time distribution - lbd", type=float, default=5.42)
    parser.add_argument("r0", help="Basic reproduction number", type=float, default=2.99)
    parser.add_argument("eta_weibull_k", help="Contamination to hospitalization interval distribution - k", type=float, default=3.6)
    parser.add_argument("eta_weibull_lbd", help="Contamination to hospitalization interval distribution - lbd", type=float, default=16.1)
    parser.add_argument("rho_exp_lbd", help="Long stay ICU interval distribution - lbd", type=float, default=1. / 16.7)
    parser.add_argument("v_exp_lbd", help="Other critical hospitalized patients interval distribution - lbd", type=float, default=1. / 6.63)
    parser.add_argument("theta_0", help="Critical illness frequency in age class [0, 24]", type=float, default=0.0006895)
    parser.add_argument("theta_1", help="Critical illness frequency in age class [25, 64]", type=float, default=0.01434299)
    parser.add_argument("theta_2", help="Critical illness frequency in age class [65, 65+]", type=float, default=0.06407823)
    parser.add_argument("psi_0", help="Long-stay ICU admission frequency in age class [0, 24]", type=float, default=0.93571031)
    parser.add_argument("psi_1", help="Long-stay ICU admission frequency in age class [25, 64]", type=float, default=0.78005851)
    parser.add_argument("psi_2", help="Long-stay ICU admission frequency in age class [65, 65+]", type=float, default=0.45428828)
    parser.add_argument("mu_0", help="Long-stay ICU fatality rate in age class [0, 24]", type=float, default=0.051)
    parser.add_argument("mu_1", help="Long-stay ICU fatality rate in age class [25, 64]", type=float, default=0.073)
    parser.add_argument("mu_2", help="Long-stay ICU fatality rate in age class [65, 65+]", type=float, default=0.194)

    # Initial conditions
    parser.add_argument("J_0_0", help="Total number of non critical infectious at time 0", type=int, default=1)
    parser.add_argument("Y_0_0", help="Total number of Critical infectious at time 0", type=int, default=0)
    parser.add_argument("H_0_0", help="Total number of Long-stay ICU hospitalized at time 0", type=int, default=0)
    parser.add_argument("W_0_0", help="Other critical hospitalized patients at time 0", type=int, default=0)
    parser.add_argument("recovered_0", help="Total number of immune at time 0", type=int, default=0)
    parser.add_argument("D_0", help="Total number of deaths at time 0", type=int, default=0)
    parser.add_argument("N_0_0", help="Total number of individuals at time 0 in age class [0, 24]", type=int, default=19980000)
    parser.add_argument("N_0_1", help="Total number of individuals at time 0 in age class [25, 64]", type=int, default=33730000)
    parser.add_argument("N_0_2", help="Total number of individuals at time 0 in age class [65, 65+]", type=int, default=13180000)

    # Simulation
    parser.add_argument("t_end", help="Simulation duration", type=int, default=196)
    parser.add_argument("t_lockdown", help="Lockdown start date", type=int, default=57)
    parser.add_argument("lockdown_contact_factor", help="Lockdown contact factor", type=float_range_0_1_type, default=0.71)
    parser.add_argument("t_deconfinement", help="Deconfinement start date", type=int, default=112)
    parser.add_argument("deconfinement_contact_factor", help="Deconfinement contact factor", type=float_range_0_1_type, default=0.91)

    args = parser.parse_args()

    # Creating the scenario
    scenario = scenario_init(simulation_duration = args.t_end,
                              lockdown_date = args.t_lockdown,
                              deconfinement_date = args.t_deconfinement,
                              lockdown_contact_factor = args.lockdown_contact_factor,
                              deconfinement_contact_factor = args.deconfinement_contact_factor)
    print(scenario)

    # Creating the age groups
    lim_age_groups = age_groups_init()

    # Package parameters
    g, h, r, u, parameters = parameters_init(r0 = args.r0,
                                 eta_weibull_k=args.eta_weibull_k,
                                 eta_weibull_lbd=args.eta_weibull_lbd,
                                 rho_exp_lbd=args.rho_exp_lbd,
                                 zeta_weibull_k=args.zeta_weibull_k,
                                 zeta_weibull_lbd=args.zeta_weibull_lbd,
                                 theta = np.array([args.theta_0, args.theta_1, args.theta_2]),
                                 psi = np.array([args.psi_0, args.psi_1, args.psi_2]),
                                 mu = np.array([args.mu_0, args.mu_1, args.mu_2]))
    print(parameters)

    # Package model architecture info
    length_init = lengths_init(g,
                               h,
                               r,
                               u)
    print(length_init)

    # Creating the object
    model = Sjyhwrd(lim_age_groups, parameters, length_init)

    # Run simulation
    initial_conditions_category = conditions_category_init(length_init = length_init,
                                                           n0 = np.array([args.N_0_0, args.N_0_1, args.N_0_2]),
                                                           j00 = args.J_0_0,
                                                           y00 = args.Y_0_0,
                                                           h00 = args.H_0_0,
                                                           w00 = args.W_0_0,
                                                           recovered_0 = args.recovered_0,
                                                           d0 = args.D_0)

    results = model.run_simulation(scenario, initial_conditions_category)

    # Plots
    _ = plot_(model, scenario['t_end'], np.transpose(results.sum(axis=2)))








