import numpy as np
import datetime

from .math_utils import diff, truncate, conditioned

# Constants : contact factors
LOCKDOWN_CONTACT_FACTOR = 0.71
DECONFINEMENT_CONTACT_FACTOR = 0.91
DEFAULT_CONTACT_FACTOR = 1.0

# Dates
DATE_FORMAT = "%Y-%m-%d"
DATE_START = "2020-01-20"
CONFINEMENT_DATE = "2020-03-17"
DECONFINEMENT_DATE = "2020-05-11"

# Model Architecture
AGE_GROUPS = [0, 25, 65]

# Simulation duration
SIMULATION_DURATION = 7*28      # ~ 7 months of simulation

# Model Parameters
ZETA_WEIBULL_K = 2.24
ZETA_WEIBULL_LBD = 5.42
RHO_EXP_LBD = 1. / 16.7
ETA_WEIBULL_K = 3.6
ETA_WEIBULL_LBD = 16.1
R_0 = 2.99
V_EXP_LBD = 1. / 6.63
THETA = np.array([0.0006895, 0.01434299, 0.06407823])
PSI = np.array([0.93571031, 0.78005851, 0.45428828])
MU = np.array([0.051, 0.073, 0.194])

# Initial conditions
N_0 = np.array([19.98, 33.73, 13.18 - 0.73]) * 1e6       # We don't take into account people in retirement homes (EHPAD)
J_0_0 = 1


def scenario_init(simulation_duration = SIMULATION_DURATION,
                  lockdown_date = 57,
                  deconfinement_date = 112,
                  lockdown_contact_factor = LOCKDOWN_CONTACT_FACTOR,
                  deconfinement_contact_factor = DECONFINEMENT_CONTACT_FACTOR):
    """Initializing scenario. The contact factors are probably the ones from the paper, but I am not sure about it."""

    def date_dist(d1, d2):
        """ Returns number of days between the two dates"""
        timedelta = datetime.datetime.strptime(d2, DATE_FORMAT) - datetime.datetime.strptime(d1, DATE_FORMAT)
        return timedelta.days

    # Temporary fix
    if (lockdown_date != 57) or (deconfinement_date):
        dates = [0,
                 lockdown_date,
                 deconfinement_date]
    else:
        dates = [0,
                 date_dist(DATE_START, CONFINEMENT_DATE),
                 date_dist(DATE_START, DECONFINEMENT_DATE)]

    return {
        't_end': simulation_duration,  # Simulation end time
        'political_measures': {
            'dates' : dates,
            'contact_factor' : [DEFAULT_CONTACT_FACTOR * np.ones(3),
                                lockdown_contact_factor * np.ones(3),
                                deconfinement_contact_factor * np.ones(3)]
        }
    }


def lengths_init(g, h, r, u):
    """Arbitrary lengths g, h, r, u."""
    return {'length_g' : g,
            'length_h' : h,
            'length_r' : r,
            'length_u' : u}


def age_groups_init():
    """Age groups"""
    return AGE_GROUPS


def parameters_init(r0 = R_0,
                    eta_weibull_k = ETA_WEIBULL_K,
                    eta_weibull_lbd = ETA_WEIBULL_LBD,
                    rho_exp_lbd = RHO_EXP_LBD,
                    zeta_weibull_k = ZETA_WEIBULL_K,
                    zeta_weibull_lbd = ZETA_WEIBULL_LBD,
                    v_exp_lbd = V_EXP_LBD,
                    theta = THETA,
                    psi = PSI,
                    mu = MU):
    zeta = diff(truncate((zeta_weibull_k, zeta_weibull_lbd), dist="Weibull"))
    eta = conditioned(truncate([eta_weibull_k, eta_weibull_lbd], dist="Weibull"))
    rho = conditioned(truncate([rho_exp_lbd], dist="Exp"))
    v = conditioned(truncate([v_exp_lbd], dist="Exp"))
    g = zeta.shape[0]
    h = eta.shape[0]
    r = rho.shape[0]
    u = v.shape[0]
    zeta_y = np.zeros(h)
    zeta_y[:min(g, h)] = zeta[:min(g, h)]

    return g, \
           h,\
           r, \
           u,\
           {'R_0': r0,  # basic reproduction number
            'zeta': zeta,
            'zeta_j': zeta,
            'zeta_y': zeta_y,
            'eta': eta,
            'rho': rho,
            'v': v,
            'theta': theta,
            'psi': psi,
            'mu': mu}


def conditions_category_init(length_init,
                             n0 = N_0,
                             j00 = J_0_0,
                             y00 = 0,
                             h00 = 0,
                             w00 = 0,
                             recovered_0 = 0,
                             d0 = 0):
    nb_age_groups = len(age_groups_init())
    ######################
    # Initial conditions
    ######################
    J_0 = np.zeros((nb_age_groups, length_init['length_g']))
    J_0[:, 0] = j00

    Y_0 = np.zeros((nb_age_groups, length_init['length_h']))
    Y_0[:, 0] = y00

    H_0 = np.zeros((nb_age_groups, length_init['length_r']))
    H_0[:, 0] = h00

    W_0 = np.zeros((nb_age_groups, length_init['length_u']))
    W_0[:, 0] = w00

    Recovered_0 = np.ones(nb_age_groups)
    Recovered_0 = Recovered_0 * recovered_0

    D_0 = np.ones(nb_age_groups)
    D_0 = D_0 * d0

    S_0 = n0 - np.sum(J_0, axis=1) - np.sum(Y_0, axis=1) - np.sum(H_0, axis=1) - np.sum(W_0, axis=1) - D_0 - Recovered_0

    return {'N_0': N_0,  # Total number of individuals at time 0
            'R_0': Recovered_0,  # Total number of immune at time 0
            'S_0': S_0,  # Total number of susceptibles at time 0
            'D_0': D_0,  # Total number of deaths at time 0
            'J_0': J_0,  # Total number of non critical infectious at time 0
            'Y_0': Y_0,  # Total number of Critical infectious at time 0
            'H_0': H_0,  # Total number of Long-stay ICU hospitalized at time 0
            'W_0': W_0,  # Other critical hospitalized patients at time 0
            }



